// SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
// SPDX-FileCopyrightText: 2024 Arjen Hiemstra <ahiemstra@heimr.nl>
//
// This file is automatically generated from property.cpp.j2.
// To regenerate, run `tools/propertygenerator/generate_properties.py`.

#include "StyleProperty.h"

#include "PropertiesTypes.h"

using namespace Union::Properties;
using namespace Qt::StringLiterals;

class Union::Properties::StylePropertyPrivate
{
public:
    std::optional<PaletteProperty> palette;
    std::optional<LayoutProperty> layout;
    std::optional<TextProperty> text;
    std::optional<IconProperty> icon;
    std::optional<BackgroundProperty> background;
};

StyleProperty::StyleProperty()
    : d(std::make_unique<StylePropertyPrivate>())
{
}

StyleProperty::StyleProperty(const StyleProperty &other)
    : d(std::make_unique<StylePropertyPrivate>())
{
    d->palette = other.d->palette;
    d->layout = other.d->layout;
    d->text = other.d->text;
    d->icon = other.d->icon;
    d->background = other.d->background;
}

StyleProperty::StyleProperty(StyleProperty &&other)
    : d(std::move(other.d))
{
}

StyleProperty::~StyleProperty() = default;

StyleProperty &StyleProperty::operator=(const StyleProperty &other)
{
    if (this != &other) {
        d->palette = other.d->palette;
        d->layout = other.d->layout;
        d->text = other.d->text;
        d->icon = other.d->icon;
        d->background = other.d->background;
    }
    return *this;
}

StyleProperty &StyleProperty::operator=(StyleProperty &&other)
{
    std::swap(d, other.d);
    return *this;
}

std::optional<PaletteProperty> StyleProperty::palette() const
{
    return d->palette;
}

void StyleProperty::setPalette(const std::optional<PaletteProperty> &newValue)
{
    if (newValue == d->palette) {
        return;
    }

    d->palette = newValue;
}
std::optional<LayoutProperty> StyleProperty::layout() const
{
    return d->layout;
}

void StyleProperty::setLayout(const std::optional<LayoutProperty> &newValue)
{
    if (newValue == d->layout) {
        return;
    }

    d->layout = newValue;
}
std::optional<TextProperty> StyleProperty::text() const
{
    return d->text;
}

void StyleProperty::setText(const std::optional<TextProperty> &newValue)
{
    if (newValue == d->text) {
        return;
    }

    d->text = newValue;
}
std::optional<IconProperty> StyleProperty::icon() const
{
    return d->icon;
}

void StyleProperty::setIcon(const std::optional<IconProperty> &newValue)
{
    if (newValue == d->icon) {
        return;
    }

    d->icon = newValue;
}
std::optional<BackgroundProperty> StyleProperty::background() const
{
    return d->background;
}

void StyleProperty::setBackground(const std::optional<BackgroundProperty> &newValue)
{
    if (newValue == d->background) {
        return;
    }

    d->background = newValue;
}

bool StyleProperty::hasAnyValue() const
{
    if (d->palette.has_value() && d->palette->hasAnyValue()) {
        return true;
    }
    if (d->layout.has_value() && d->layout->hasAnyValue()) {
        return true;
    }
    if (d->text.has_value() && d->text->hasAnyValue()) {
        return true;
    }
    if (d->icon.has_value() && d->icon->hasAnyValue()) {
        return true;
    }
    if (d->background.has_value() && d->background->hasAnyValue()) {
        return true;
    }
    return false;
}

void StyleProperty::resolveProperties(const StyleProperty &source, StyleProperty &destination)
{
    if (source.d->palette.has_value()) {
        PaletteProperty value;
        if (destination.d->palette.has_value()) {
            value = destination.d->palette.value();
        }
        PaletteProperty::resolveProperties(source.d->palette.value(), value);
        if (value.hasAnyValue()) {
            destination.d->palette = value;
        }
    }
    if (source.d->layout.has_value()) {
        LayoutProperty value;
        if (destination.d->layout.has_value()) {
            value = destination.d->layout.value();
        }
        LayoutProperty::resolveProperties(source.d->layout.value(), value);
        if (value.hasAnyValue()) {
            destination.d->layout = value;
        }
    }
    if (source.d->text.has_value()) {
        TextProperty value;
        if (destination.d->text.has_value()) {
            value = destination.d->text.value();
        }
        TextProperty::resolveProperties(source.d->text.value(), value);
        if (value.hasAnyValue()) {
            destination.d->text = value;
        }
    }
    if (source.d->icon.has_value()) {
        IconProperty value;
        if (destination.d->icon.has_value()) {
            value = destination.d->icon.value();
        }
        IconProperty::resolveProperties(source.d->icon.value(), value);
        if (value.hasAnyValue()) {
            destination.d->icon = value;
        }
    }
    if (source.d->background.has_value()) {
        BackgroundProperty value;
        if (destination.d->background.has_value()) {
            value = destination.d->background.value();
        }
        BackgroundProperty::resolveProperties(source.d->background.value(), value);
        if (value.hasAnyValue()) {
            destination.d->background = value;
        }
    }
}

StyleProperty StyleProperty::empty()
{
    StyleProperty result;
    result.d->palette = PaletteProperty::empty();
    result.d->layout = LayoutProperty::empty();
    result.d->text = TextProperty::empty();
    result.d->icon = IconProperty::empty();
    result.d->background = BackgroundProperty::empty();
    return result;
}

bool Union::Properties::operator==(const StyleProperty &left, const StyleProperty &right)
{
    if (left.palette() != right.palette()) {
        return false;
    }
    if (left.layout() != right.layout()) {
        return false;
    }
    if (left.text() != right.text()) {
        return false;
    }
    if (left.icon() != right.icon()) {
        return false;
    }
    if (left.background() != right.background()) {
        return false;
    }
    return true;
}

QDebug operator<<(QDebug debug, const Union::Properties::StyleProperty &type)
{
    QDebugStateSaver saver(debug);
    debug.nospace() << "StyleProperty(" //
                    << "palette: " << type.palette() //
                    << ", layout: " << type.layout() //
                    << ", text: " << type.text() //
                    << ", icon: " << type.icon() //
                    << ", background: " << type.background() //
                    << ")";
    return debug;
}